/**
 * OW2 Util
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EncodingType.java 5405 2010-03-11 16:24:57Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.auditreport.impl;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.ow2.util.auditreport.api.IDisplayableType;
import org.ow2.util.auditreport.api.IEncodableType;

/**
 * This class allow to encode complex type.
 * @author Vincent Michaud
 */
public final class EncodingType {

    /**
     * The constructor.
     */
    private EncodingType() {
    }

    /**
     * Regular expression for stacktrace values.
     */
    private static final Pattern STACKTRACE_PATTERN = Pattern.compile("([^(]+)\\(([^:]+):([0-9]+)");

    /**
     * Displayable stacktrace.
     */
    public static final IDisplayableType<StackTraceElement[]> STACKTRACE = new IDisplayableType<StackTraceElement[]>() {

        /**
         * Convert an object into a string.
         * @param stackTrace The stacktrace
         * @return A string representation of the object.
         */
        public String toString(final StackTraceElement[] stackTrace) {
            StringBuilder builder = new StringBuilder();
            for (StackTraceElement elmt : stackTrace) {
                builder.append(elmt.getClassName());
                builder.append('#');
                builder.append(elmt.getMethodName());
                builder.append('#');
                builder.append(elmt.getFileName());
                builder.append('#');
                builder.append(elmt.getLineNumber());
                builder.append('\n');
            }
            return builder.toString();
        }

        /**
         * Create a new object from a string.
         * @param stackTrace The stcktrace
         * @return The new object
         */
        public StackTraceElement[] fromString(final String stackTrace) {
            String[] values, traces = stackTrace.split("\n");
            StackTraceElement[] elmts = new StackTraceElement[traces.length];
            for (int i = 0; i < elmts.length; i++) {
                values = traces[i].split("#");
                elmts[i] = new StackTraceElement(values[0], values[1], values[2], Integer.parseInt(values[3]));
            }
            return elmts;
        }

        /**
         * Convert an encoded string to a displayable string.
         * @param encodedString The encoded string
         * @return A string representation of the object, or a list of string
         *         (in case of array, or complex objects) Each string must not
         *         contain new line characters. Returning a list of string can
         *         be used for this. Also, each string must not begin with a
         *         sequence of space.
         */
        public Object encodedToDisplayable(final String encodedStackTrace) {
            String[] values, traces = encodedStackTrace.split("\n");
            List<String> result = new LinkedList<String>();
            StringBuilder builder = new StringBuilder();
            for (int i = 0; i < traces.length; i++) {
                values = traces[i].split("#");
                builder.append(values[0]);
                builder.append('.');
                builder.append(values[1]);
                builder.append('(');
                builder.append(values[2]);
                builder.append(':');
                builder.append(values[3]);
                builder.append(')');
                result.add(builder.toString());
                builder.setLength(0);
            }
            return result;
        }

        /**
         * Convert a displayable string to an encoded string.
         * @param strObject The displayble object. The object can be a string or
         *        a list of string (in case of array, or complex objects). Each
         *        string does not contain new line characters.
         * @return The new object
         */
        @SuppressWarnings("unchecked")
        public String displayableToEncoded(final Object strObject) {
            List<String> stacktraces = (List<String>) strObject;
            StringBuilder builder = new StringBuilder();
            Matcher matcher;
            for (String stacktrace : stacktraces) {
                matcher = STACKTRACE_PATTERN.matcher(stacktrace);
                if (matcher.find()) {
                    String function = matcher.group(0);
                    int index = function.lastIndexOf(".");
                    builder.append(function.substring(0, index));
                    builder.append('#');
                    builder.append(function.substring(index + 1));
                    builder.append('#');
                    builder.append(matcher.group(1));
                    builder.append('#');
                    builder.append(Integer.parseInt(matcher.group(2)));
                    builder.append('\n');
                }
            }
            return builder.toString();
        }
    };

    /**
     * Displayable table.
     */
    public static final IEncodableType<Object[]> OBJECTTABLE = new IEncodableType<Object[]>() {

        public String toString(final Object[] object) {
            StringBuilder builder = new StringBuilder();
            builder.append('[');
            for (Object elmt : object) {
                if (elmt != null) {
                    builder.append(elmt.toString());
                    builder.append(" - ");
                }
            }
            if (builder.length() >= 3) {
                builder.setLength(builder.length() - 3);
            }
            builder.append(']');
            return builder.toString();
        }

        public Object[] fromString(final String object) {
            String tmp = object.replace("[", "");
            tmp = tmp.replace("]", "");
            return tmp.split(" - ");
        }
    };

    /**
     * Displayable table.
     */
    public static final IEncodableType<List> LIST = new IEncodableType<List>() {

        public String toString(final List list) {
            StringBuilder builder = new StringBuilder();
            builder.append('[');
            if (list != null) {
                for (Object elmt : list) {
                    if (elmt != null) {
                        builder.append(elmt.toString());
                        builder.append(" - ");
                    }
                }
            }
            if (builder.length() >= 3) {
                builder.setLength(builder.length() - 3);
            }
            builder.append(']');
            return builder.toString();
        }

        public List fromString(final String object) {
            String tmp = object.replace("[", "");
            tmp = tmp.replace("]", "");
            return Arrays.asList(tmp.split(" - "));
        }
    };

    /**
     * Displayable object.
     */
    public static final IEncodableType<Object> OBJECT = new IEncodableType<Object>() {

        public String toString(final Object object) {
            return object.toString();
        }

        public Object fromString(final String object) {
            return object;
        }
    };

    /**
     * Displayable ENUM.
     */
    public static final IEncodableType<Object> LCENUM = new IEncodableType<Object>() {

        public String toString(final Object object) {
            return object.toString();
        }

        public LifeCycleAuditReport.STEP fromString(final String object) {
            if (object.equals("STARTING")) {
                return LifeCycleAuditReport.STEP.STARTING;
            } else if (object.equals("STARTED")) {
                return LifeCycleAuditReport.STEP.STARTED;
            } else if (object.equals("STOPPING")) {
                return LifeCycleAuditReport.STEP.STOPPING;
            } else {
                return LifeCycleAuditReport.STEP.STOPPED;
            }
        }
    };
}
